package ch.std.rest.upload.test;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.util.LinkedMultiValueMap;

import ch.std.rest.upload.dto.RequestUploadDTO;
import ch.std.rest.upload.dto.ResponseUploadDTO;

@ExtendWith(SpringExtension.class)
@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT)
public class UploadServiceIntegrationTests {

	@Autowired
	private TestRestTemplate restTemplate;

	@Test
	public void contextLoads() {
	}

	@Test
	public void testUploadController() throws Exception {
		var multipart = new LinkedMultiValueMap<>();
		multipart.add("file", new org.springframework.core.io.ClassPathResource("dummy.pdf"));

		final ResponseEntity<ResponseUploadDTO> post = this.restTemplate.postForEntity("/rest/upload",
				new HttpEntity<>(multipart, headers()), ResponseUploadDTO.class);
		
		assertEquals(HttpStatus.OK, post.getStatusCode());
		System.out.println("response = " + post.getBody());
	}

	@Test
	public void testUploadDTOController() throws Exception {
		var multipart = new LinkedMultiValueMap<>();
		RequestUploadDTO requestUploadDTO = new RequestUploadDTO();
		requestUploadDTO.setFileName("dummydto.pdf");
		multipart.add("dto", requestUploadDTO);
		multipart.add("file", new org.springframework.core.io.ClassPathResource("dummy.pdf"));

		final ResponseEntity<ResponseUploadDTO> post = this.restTemplate.postForEntity("/rest/uploaddto",
				new HttpEntity<>(multipart, headers()), ResponseUploadDTO.class);
		
		assertEquals(HttpStatus.OK, post.getStatusCode());
		System.out.println("response = " + post.getBody());
	}

	private HttpHeaders headers() {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.MULTIPART_FORM_DATA);
		return headers;
	}
}
